@@ -1,7 +1,6 @@
 use crate::git::repo;
 use crate::paths;
-use cargo::sources::CRATES_IO_INDEX;
-use cargo::util::Sha256;
+use cargo_util::Sha256;
 use flate2::write::GzEncoder;
 use flate2::Compression;
 use std::collections::BTreeMap;
@@ -193,7 +192,7 @@ impl RegistryBuilder {
                 alt_dl_url(),
                 self.alt_api_url
                     .as_ref()
-                    .map_or_else(alt_api_url, |url| Url::parse(&url).expect("valid url")),
+                    .map_or_else(alt_api_url, |url| Url::parse(url).expect("valid url")),
                 alt_api_path(),
             );
         }
@@ -560,7 +559,7 @@ impl Package {
 
     /// Sets the index schema version for this package.
     ///
-    /// See [`cargo::sources::registry::RegistryPackage`] for more information.
+    /// See `cargo::sources::registry::RegistryPackage` for more information.
     pub fn schema_version(&mut self, version: u32) -> &mut Package {
         self.v = Some(version);
         self
@@ -585,7 +584,9 @@ impl Package {
                 let registry_url = match (self.alternative, dep.registry.as_deref()) {
                     (false, None) => None,
                     (false, Some("alternative")) => Some(alt_registry_url().to_string()),
-                    (true, None) => Some(CRATES_IO_INDEX.to_string()),
+                    (true, None) => {
+                        Some("https://github.com/rust-lang/crates.io-index".to_string())
+                    }
                     (true, Some("alternative")) => None,
                     _ => panic!("registry_dep currently only supports `alternative`"),
                 };
