@@ -7,6 +7,7 @@ use cargo::ops;
 pub fn cli() -> App {
     subcommand("search")
         .about("Search packages in crates.io")
+        .arg(opt("quiet", "No output printed to stdout").short("q"))
         .arg(Arg::with_name("query").multiple(true))
         .arg_index()
         .arg(
@@ -17,6 +18,7 @@ pub fn cli() -> App {
             .value_name("LIMIT"),
         )
         .arg(opt("registry", "Registry to use").value_name("REGISTRY"))
+        .after_help("Run `cargo help search` for more detailed information.\n")
 }
 
 pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
