@@ -56,7 +56,8 @@ use std::marker;
 use std::sync::Arc;
 use std::time::Duration;
 
-use anyhow::format_err;
+use anyhow::{format_err, Context as _};
+use cargo_util::ProcessBuilder;
 use crossbeam_utils::thread::Scope;
 use jobserver::{Acquired, Client, HelperThread};
 use log::{debug, info, trace};
@@ -77,8 +78,8 @@ use crate::core::{PackageId, Shell, TargetKind};
 use crate::drop_eprint;
 use crate::util::diagnostic_server::{self, DiagnosticPrinter};
 use crate::util::machine_message::{self, Message as _};
+use crate::util::CargoResult;
 use crate::util::{self, internal, profile};
-use crate::util::{CargoResult, CargoResultExt, ProcessBuilder};
 use crate::util::{Config, DependencyQueue, Progress, ProgressStyle, Queue};
 
 /// This structure is backed by the `DependencyQueue` type and manages the
@@ -439,7 +440,7 @@ impl<'cfg> JobQueue<'cfg> {
             .into_helper_thread(move |token| {
                 messages.push(Message::Token(token));
             })
-            .chain_err(|| "failed to create helper thread for jobserver management")?;
+            .with_context(|| "failed to create helper thread for jobserver management")?;
 
         // Create a helper thread to manage the diagnostics for rustfix if
         // necessary.
@@ -536,7 +537,7 @@ impl<'cfg> DrainState<'cfg> {
                 .push(token);
             client
                 .release_raw()
-                .chain_err(|| "failed to release jobserver token")?;
+                .with_context(|| "failed to release jobserver token")?;
         }
 
         Ok(())
@@ -616,7 +617,7 @@ impl<'cfg> DrainState<'cfg> {
                     .push(FutureIncompatReportCrate { package_id, report });
             }
             Message::Token(acquired_token) => {
-                let token = acquired_token.chain_err(|| "failed to acquire jobserver token")?;
+                let token = acquired_token.with_context(|| "failed to acquire jobserver token")?;
                 self.tokens.push(token);
             }
             Message::NeedsToken(id) => {
@@ -807,9 +808,16 @@ impl<'cfg> DrainState<'cfg> {
     }
 
     fn emit_future_incompat(&mut self, cx: &mut Context<'_, '_>) {
-        if cx.bcx.config.cli_unstable().enable_future_incompat_feature
-            && !self.per_crate_future_incompat_reports.is_empty()
-        {
+        if cx.bcx.config.cli_unstable().future_incompat_report {
+            if self.per_crate_future_incompat_reports.is_empty() {
+                drop(
+                    cx.bcx
+                        .config
+                        .shell()
+                        .note("0 dependencies had future-incompat warnings"),
+                );
+                return;
+            }
             self.per_crate_future_incompat_reports
                 .sort_by_key(|r| r.package_id);
 
