@@ -26,11 +26,10 @@ use std::collections::{BTreeSet, HashSet};
 use std::io::{BufWriter, Write};
 use std::path::{Path, PathBuf};
 
-use log::debug;
-
 use super::{fingerprint, Context, FileFlavor, Unit};
-use crate::util::paths;
 use crate::util::{internal, CargoResult};
+use cargo_util::paths;
+use log::debug;
 
 fn render_filename<P: AsRef<Path>>(path: P, basedir: Option<&str>) -> CargoResult<String> {
     let path = path.as_ref();
