@@ -5,23 +5,24 @@ use std::path::{Path, PathBuf};
 use std::rc::Rc;
 use std::slice;
 
+use anyhow::{bail, Context as _};
 use glob::glob;
 use log::debug;
 use url::Url;
 
 use crate::core::features::Features;
 use crate::core::registry::PackageRegistry;
-use crate::core::resolver::features::RequestedFeatures;
+use crate::core::resolver::features::CliFeatures;
 use crate::core::resolver::ResolveBehavior;
-use crate::core::{Dependency, PackageId, PackageIdSpec};
+use crate::core::{Dependency, Edition, FeatureValue, PackageId, PackageIdSpec};
 use crate::core::{EitherManifest, Package, SourceId, VirtualManifest};
 use crate::ops;
-use crate::sources::PathSource;
-use crate::util::errors::{CargoResult, CargoResultExt, ManifestError};
+use crate::sources::{PathSource, CRATES_IO_INDEX, CRATES_IO_REGISTRY};
+use crate::util::errors::{CargoResult, ManifestError};
 use crate::util::interning::InternedString;
-use crate::util::paths;
-use crate::util::toml::{read_manifest, TomlProfiles};
-use crate::util::{Config, Filesystem};
+use crate::util::toml::{read_manifest, TomlDependency, TomlProfiles};
+use crate::util::{config::ConfigRelativePath, Config, Filesystem, IntoUrl};
+use cargo_util::paths;
 
 /// The core abstraction in Cargo for working with a workspace of crates.
 ///
@@ -88,7 +89,7 @@ pub struct Workspace<'cfg> {
     ignore_lock: bool,
 
     /// The resolver behavior specified with the `resolver` field.
-    resolve_behavior: Option<ResolveBehavior>,
+    resolve_behavior: ResolveBehavior,
 
     /// Workspace-level custom metadata
     custom_metadata: Option<toml::Value>,
@@ -103,7 +104,7 @@ struct Packages<'cfg> {
 }
 
 #[derive(Debug)]
-enum MaybePackage {
+pub enum MaybePackage {
     Package(Package),
     Virtual(VirtualManifest),
 }
@@ -152,7 +153,7 @@ impl<'cfg> Workspace<'cfg> {
         ws.target_dir = config.target_dir()?;
 
         if manifest_path.is_relative() {
-            anyhow::bail!(
+            bail!(
                 "manifest_path:{:?} is not an absolute path. Please provide an absolute path.",
                 manifest_path
             )
@@ -164,10 +165,7 @@ impl<'cfg> Workspace<'cfg> {
             .load_workspace_config()?
             .and_then(|cfg| cfg.custom_metadata);
         ws.find_members()?;
-        ws.resolve_behavior = match ws.root_maybe() {
-            MaybePackage::Package(p) => p.manifest().resolve_behavior(),
-            MaybePackage::Virtual(vm) => vm.resolve_behavior(),
-        };
+        ws.set_resolve_behavior();
         ws.validate()?;
         Ok(ws)
     }
@@ -189,7 +187,7 @@ impl<'cfg> Workspace<'cfg> {
             require_optional_deps: true,
             loaded_packages: RefCell::new(HashMap::new()),
             ignore_lock: false,
-            resolve_behavior: None,
+            resolve_behavior: ResolveBehavior::V1,
             custom_metadata: None,
         }
     }
@@ -203,11 +201,11 @@ impl<'cfg> Workspace<'cfg> {
         let mut ws = Workspace::new_default(current_manifest, config);
         ws.root_manifest = Some(root_path.join("Cargo.toml"));
         ws.target_dir = config.target_dir()?;
-        ws.resolve_behavior = manifest.resolve_behavior();
         ws.packages
             .packages
             .insert(root_path, MaybePackage::Virtual(manifest));
         ws.find_members()?;
+        ws.set_resolve_behavior();
         // TODO: validation does not work because it walks up the directory
         // tree looking for the root which is a fake file that doesn't exist.
         Ok(ws)
@@ -231,7 +229,6 @@ impl<'cfg> Workspace<'cfg> {
         let mut ws = Workspace::new_default(package.manifest_path().to_path_buf(), config);
         ws.is_ephemeral = true;
         ws.require_optional_deps = require_optional_deps;
-        ws.resolve_behavior = package.manifest().resolve_behavior();
         let key = ws.current_manifest.parent().unwrap();
         let id = package.package_id();
         let package = MaybePackage::Package(package);
@@ -244,9 +241,28 @@ impl<'cfg> Workspace<'cfg> {
         ws.members.push(ws.current_manifest.clone());
         ws.member_ids.insert(id);
         ws.default_members.push(ws.current_manifest.clone());
+        ws.set_resolve_behavior();
         Ok(ws)
     }
 
+    fn set_resolve_behavior(&mut self) {
+        // - If resolver is specified in the workspace definition, use that.
+        // - If the root package specifies the resolver, use that.
+        // - If the root package specifies edition 2021, use v2.
+        // - Otherwise, use the default v1.
+        self.resolve_behavior = match self.root_maybe() {
+            MaybePackage::Package(p) => p.manifest().resolve_behavior().or_else(|| {
+                if p.manifest().edition() >= Edition::Edition2021 {
+                    Some(ResolveBehavior::V2)
+                } else {
+                    None
+                }
+            }),
+            MaybePackage::Virtual(vm) => vm.resolve_behavior(),
+        }
+        .unwrap_or(ResolveBehavior::V1);
+    }
+
     /// Returns the current package of this workspace.
     ///
     /// Note that this can return an error if it the current manifest is
@@ -327,7 +343,7 @@ impl<'cfg> Workspace<'cfg> {
     }
 
     /// Returns the root Package or VirtualManifest.
-    fn root_maybe(&self) -> &MaybePackage {
+    pub fn root_maybe(&self) -> &MaybePackage {
         self.packages.get(self.root_manifest())
     }
 
@@ -347,14 +363,105 @@ impl<'cfg> Workspace<'cfg> {
         }
     }
 
+    fn config_patch(&self) -> CargoResult<HashMap<Url, Vec<Dependency>>> {
+        let config_patch: Option<
+            BTreeMap<String, BTreeMap<String, TomlDependency<ConfigRelativePath>>>,
+        > = self.config.get("patch")?;
+
+        if config_patch.is_some() && !self.config.cli_unstable().patch_in_config {
+            self.config.shell().warn("`[patch]` in cargo config was ignored, the -Zpatch-in-config command-line flag is required".to_owned())?;
+            return Ok(HashMap::new());
+        }
+
+        let source = SourceId::for_path(self.root())?;
+
+        let mut warnings = Vec::new();
+        let mut nested_paths = Vec::new();
+
+        let mut patch = HashMap::new();
+        for (url, deps) in config_patch.into_iter().flatten() {
+            let url = match &url[..] {
+                CRATES_IO_REGISTRY => CRATES_IO_INDEX.parse().unwrap(),
+                url => self
+                    .config
+                    .get_registry_index(url)
+                    .or_else(|_| url.into_url())
+                    .with_context(|| {
+                        format!("[patch] entry `{}` should be a URL or registry name", url)
+                    })?,
+            };
+            patch.insert(
+                url,
+                deps.iter()
+                    .map(|(name, dep)| {
+                        dep.to_dependency_split(
+                            name,
+                            /* pkg_id */ None,
+                            source,
+                            &mut nested_paths,
+                            self.config,
+                            &mut warnings,
+                            /* platform */ None,
+                            // NOTE: Since we use ConfigRelativePath, this root isn't used as
+                            // any relative paths are resolved before they'd be joined with root.
+                            Path::new("unused-relative-path"),
+                            self.unstable_features(),
+                            /* kind */ None,
+                        )
+                    })
+                    .collect::<CargoResult<Vec<_>>>()?,
+            );
+        }
+
+        for message in warnings {
+            self.config
+                .shell()
+                .warn(format!("[patch] in cargo config: {}", message))?
+        }
+
+        Ok(patch)
+    }
+
     /// Returns the root `[patch]` section of this workspace.
     ///
     /// This may be from a virtual crate or an actual crate.
-    pub fn root_patch(&self) -> &HashMap<Url, Vec<Dependency>> {
-        match self.root_maybe() {
+    pub fn root_patch(&self) -> CargoResult<HashMap<Url, Vec<Dependency>>> {
+        let from_manifest = match self.root_maybe() {
             MaybePackage::Package(p) => p.manifest().patch(),
             MaybePackage::Virtual(vm) => vm.patch(),
+        };
+
+        let from_config = self.config_patch()?;
+        if from_config.is_empty() {
+            return Ok(from_manifest.clone());
         }
+        if from_manifest.is_empty() {
+            return Ok(from_config);
+        }
+
+        // We could just chain from_manifest and from_config,
+        // but that's not quite right as it won't deal with overlaps.
+        let mut combined = from_manifest.clone();
+        for (url, cdeps) in from_config {
+            if let Some(deps) = combined.get_mut(&url) {
+                // We want from_manifest to take precedence for each patched name.
+                // NOTE: This is inefficient if the number of patches is large!
+                let mut left = cdeps.clone();
+                for dep in &mut *deps {
+                    if let Some(i) = left.iter().position(|cdep| {
+                        // XXX: should this also take into account version numbers?
+                        dep.name_in_toml() == cdep.name_in_toml()
+                    }) {
+                        left.swap_remove(i);
+                    }
+                }
+                // Whatever is left does not exist in manifest dependencies.
+                deps.extend(left);
+            } else {
+                combined.insert(url.clone(), cdeps.clone());
+            }
+        }
+        Ok(combined)
     }
 
     /// Returns an iterator over all packages in this workspace
@@ -417,7 +524,7 @@ impl<'cfg> Workspace<'cfg> {
                     return Ok(Some(root_config.clone()));
                 }
 
-                _ => anyhow::bail!(
+                _ => bail!(
                     "root of a workspace inferred but wasn't a root: {}",
                     root_path.display()
                 ),
@@ -437,14 +544,14 @@ impl<'cfg> Workspace<'cfg> {
     /// Returns an error if `manifest_path` isn't actually a valid manifest or
     /// if some other transient error happens.
     fn find_root(&mut self, manifest_path: &Path) -> CargoResult<Option<PathBuf>> {
-        fn read_root_pointer(member_manifest: &Path, root_link: &str) -> CargoResult<PathBuf> {
+        fn read_root_pointer(member_manifest: &Path, root_link: &str) -> PathBuf {
             let path = member_manifest
                 .parent()
                 .unwrap()
                 .join(root_link)
                 .join("Cargo.toml");
             debug!("find_root - pointer {}", path.display());
-            Ok(paths::normalize_path(&path))
+            paths::normalize_path(&path)
         }
 
         {
@@ -456,12 +563,12 @@ impl<'cfg> Workspace<'cfg> {
                 }
                 WorkspaceConfig::Member {
                     root: Some(ref path_to_root),
-                } => return Ok(Some(read_root_pointer(manifest_path, path_to_root)?)),
+                } => return Ok(Some(read_root_pointer(manifest_path, path_to_root))),
                 WorkspaceConfig::Member { root: None } => {}
             }
         }
 
-        for path in paths::ancestors(manifest_path).skip(2) {
+        for path in paths::ancestors(manifest_path, None).skip(2) {
             if path.ends_with("target/package") {
                 break;
             }
@@ -481,7 +588,7 @@ impl<'cfg> Workspace<'cfg> {
                         root: Some(ref path_to_root),
                     } => {
                         debug!("find_root - found pointer");
-                        return Ok(Some(read_root_pointer(&ances_manifest_path, path_to_root)?));
+                        return Ok(Some(read_root_pointer(&ances_manifest_path, path_to_root)));
                     }
                     WorkspaceConfig::Member { .. } => {}
                 }
@@ -538,7 +645,13 @@ impl<'cfg> Workspace<'cfg> {
         };
 
         for path in &members_paths {
-            self.find_path_deps(&path.join("Cargo.toml"), &root_manifest_path, false)?;
+            self.find_path_deps(&path.join("Cargo.toml"), &root_manifest_path, false)
+                .with_context(|| {
+                    format!(
+                        "failed to load manifest for workspace member `{}`",
+                        path.display()
+                    )
+                })?;
         }
 
         if let Some(default) = default_members_paths {
@@ -557,7 +670,7 @@ impl<'cfg> Workspace<'cfg> {
                     if exclude {
                         continue;
                     }
-                    anyhow::bail!(
+                    bail!(
                         "package `{}` is listed in workspace’s default-members \
                          but is not a member.",
                         path.display()
@@ -612,14 +725,15 @@ impl<'cfg> Workspace<'cfg> {
             self.member_ids.insert(pkg.package_id());
             pkg.dependencies()
                 .iter()
-                .map(|d| d.source_id())
-                .filter(|d| d.is_path())
-                .filter_map(|d| d.url().to_file_path().ok())
-                .map(|p| p.join("Cargo.toml"))
+                .map(|d| (d.source_id(), d.package_name()))
+                .filter(|(s, _)| s.is_path())
+                .filter_map(|(s, n)| s.url().to_file_path().ok().map(|p| (p, n)))
+                .map(|(p, n)| (p.join("Cargo.toml"), n))
                 .collect::<Vec<_>>()
         };
-        for candidate in candidates {
-            self.find_path_deps(&candidate, root_manifest, true)
+        for (path, name) in candidates {
+            self.find_path_deps(&path, root_manifest, true)
+                .with_context(|| format!("failed to load manifest for dependency `{}`", name))
                 .map_err(|err| ManifestError::new(err, manifest_path.clone()))?;
         }
         Ok(())
@@ -634,11 +748,18 @@ impl<'cfg> Workspace<'cfg> {
     }
 
     pub fn resolve_behavior(&self) -> ResolveBehavior {
-        self.resolve_behavior.unwrap_or(ResolveBehavior::V1)
+        self.resolve_behavior
     }
 
-    pub fn allows_unstable_package_features(&self) -> bool {
-        self.config().cli_unstable().package_features
+    /// Returns `true` if this workspace uses the new CLI features behavior.
+    ///
+    /// The old behavior only allowed choosing the features from the package
+    /// in the current directory, regardless of which packages were chosen
+    /// with the -p flags. The new behavior allows selecting features from the
+    /// packages chosen on the command line (with -p or --workspace flags),
+    /// ignoring whatever is in the current directory.
+    pub fn allows_new_cli_feature_behavior(&self) -> bool {
+        self.is_virtual()
             || match self.resolve_behavior() {
                 ResolveBehavior::V1 => false,
                 ResolveBehavior::V2 => true,
@@ -672,7 +793,7 @@ impl<'cfg> Workspace<'cfg> {
                 MaybePackage::Virtual(_) => continue,
             };
             if let Some(prev) = names.insert(name, member) {
-                anyhow::bail!(
+                bail!(
                     "two packages named `{}` in this workspace:\n\
                          - {}\n\
                          - {}",
@@ -697,7 +818,7 @@ impl<'cfg> Workspace<'cfg> {
             .collect();
         match roots.len() {
             1 => Ok(()),
-            0 => anyhow::bail!(
+            0 => bail!(
                 "`package.workspace` configuration points to a crate \
                  which is not configured with [workspace]: \n\
                  configuration at: {}\n\
@@ -706,7 +827,7 @@ impl<'cfg> Workspace<'cfg> {
                 self.root_manifest.as_ref().unwrap().display()
             ),
             _ => {
-                anyhow::bail!(
+                bail!(
                     "multiple workspace roots found in the same workspace:\n{}",
                     roots
                         .iter()
@@ -727,7 +848,7 @@ impl<'cfg> Workspace<'cfg> {
 
             match root {
                 Some(root) => {
-                    anyhow::bail!(
+                    bail!(
                         "package `{}` is a member of the wrong workspace\n\
                          expected: {}\n\
                          actual:   {}",
@@ -737,7 +858,7 @@ impl<'cfg> Workspace<'cfg> {
                     );
                 }
                 None => {
-                    anyhow::bail!(
+                    bail!(
                         "workspace member `{}` is not hierarchically below \
                          the workspace root `{}`",
                         member.display(),
@@ -794,7 +915,7 @@ impl<'cfg> Workspace<'cfg> {
                 }
             }
         };
-        anyhow::bail!(
+        bail!(
             "current package believes it's in a workspace when it's not:\n\
                  current:   {}\n\
                  workspace: {}\n\n{}\n\
@@ -836,11 +957,11 @@ impl<'cfg> Workspace<'cfg> {
                 if !manifest.patch().is_empty() {
                     emit_warning("patch")?;
                 }
-                if manifest.resolve_behavior().is_some()
-                    && manifest.resolve_behavior() != self.resolve_behavior
-                {
-                    // Only warn if they don't match.
-                    emit_warning("resolver")?;
+                if let Some(behavior) = manifest.resolve_behavior() {
+                    if behavior != self.resolve_behavior {
+                        // Only warn if they don't match.
+                        emit_warning("resolver")?;
+                    }
                 }
             }
         }
@@ -850,7 +971,7 @@ impl<'cfg> Workspace<'cfg> {
     pub fn load(&self, manifest_path: &Path) -> CargoResult<Package> {
         match self.packages.maybe_get(manifest_path) {
             Some(&MaybePackage::Package(ref p)) => return Ok(p.clone()),
-            Some(&MaybePackage::Virtual(_)) => anyhow::bail!("cannot load workspace root"),
+            Some(&MaybePackage::Virtual(_)) => bail!("cannot load workspace root"),
             None => {}
         }
 
@@ -933,10 +1054,10 @@ impl<'cfg> Workspace<'cfg> {
     pub fn members_with_features(
         &self,
         specs: &[PackageIdSpec],
-        requested_features: &RequestedFeatures,
-    ) -> CargoResult<Vec<(&Package, RequestedFeatures)>> {
+        cli_features: &CliFeatures,
+    ) -> CargoResult<Vec<(&Package, CliFeatures)>> {
         assert!(
-            !specs.is_empty() || requested_features.all_features,
+            !specs.is_empty() || cli_features.all_features,
             "no specs requires all_features"
         );
         if specs.is_empty() {
@@ -944,31 +1065,32 @@ impl<'cfg> Workspace<'cfg> {
             // all features enabled.
             return Ok(self
                 .members()
-                .map(|m| (m, RequestedFeatures::new_all(true)))
+                .map(|m| (m, CliFeatures::new_all(true)))
                 .collect());
         }
-        if self.allows_unstable_package_features() {
-            self.members_with_features_pf(specs, requested_features)
+        if self.allows_new_cli_feature_behavior() {
+            self.members_with_features_new(specs, cli_features)
         } else {
-            self.members_with_features_stable(specs, requested_features)
+            Ok(self.members_with_features_old(specs, cli_features))
         }
     }
 
-    /// New command-line feature selection with -Zpackage-features.
-    fn members_with_features_pf(
+    /// New command-line feature selection behavior with resolver = "2" or the
+    /// root of a virtual workspace. See `allows_new_cli_feature_behavior`.
+    fn members_with_features_new(
         &self,
         specs: &[PackageIdSpec],
-        requested_features: &RequestedFeatures,
-    ) -> CargoResult<Vec<(&Package, RequestedFeatures)>> {
+        cli_features: &CliFeatures,
+    ) -> CargoResult<Vec<(&Package, CliFeatures)>> {
         // Keep track of which features matched *any* member, to produce an error
         // if any of them did not match anywhere.
-        let mut found: BTreeSet<InternedString> = BTreeSet::new();
+        let mut found: BTreeSet<FeatureValue> = BTreeSet::new();
 
         // Returns the requested features for the given member.
         // This filters out any named features that the member does not have.
-        let mut matching_features = |member: &Package| -> RequestedFeatures {
-            if requested_features.features.is_empty() || requested_features.all_features {
-                return requested_features.clone();
+        let mut matching_features = |member: &Package| -> CliFeatures {
+            if cli_features.features.is_empty() || cli_features.all_features {
+                return cli_features.clone();
             }
             // Only include features this member defines.
             let summary = member.summary();
@@ -984,40 +1106,54 @@ impl<'cfg> Workspace<'cfg> {
                         .any(|dep| dep.is_optional() && dep.name_in_toml() == feature)
             };
 
-            for feature in requested_features.features.iter() {
-                let mut split = feature.splitn(2, '/');
-                let split = (split.next().unwrap(), split.next());
-                if let (pkg, Some(pkg_feature)) = split {
-                    let pkg = InternedString::new(pkg);
-                    let pkg_feature = InternedString::new(pkg_feature);
-                    if summary
-                        .dependencies()
-                        .iter()
-                        .any(|dep| dep.name_in_toml() == pkg)
-                    {
-                        // pkg/feat for a dependency.
-                        // Will rely on the dependency resolver to validate `feat`.
-                        features.insert(*feature);
-                        found.insert(*feature);
-                    } else if pkg == member.name() && contains(pkg_feature) {
-                        // member/feat where "feat" is a feature in member.
-                        features.insert(pkg_feature);
-                        found.insert(*feature);
+            for feature in cli_features.features.iter() {
+                match feature {
+                    FeatureValue::Feature(f) => {
+                        if contains(*f) {
+                            // feature exists in this member.
+                            features.insert(feature.clone());
+                            found.insert(feature.clone());
+                        }
+                    }
+                    // This should be enforced by CliFeatures.
+                    FeatureValue::Dep { .. }
+                    | FeatureValue::DepFeature {
+                        dep_prefix: true, ..
+                    } => panic!("unexpected dep: syntax {}", feature),
+                    FeatureValue::DepFeature {
+                        dep_name,
+                        dep_feature,
+                        dep_prefix: _,
+                        weak: _,
+                    } => {
+                        if summary
+                            .dependencies()
+                            .iter()
+                            .any(|dep| dep.name_in_toml() == *dep_name)
+                        {
+                            // pkg/feat for a dependency.
+                            // Will rely on the dependency resolver to validate `dep_feature`.
+                            features.insert(feature.clone());
+                            found.insert(feature.clone());
+                        } else if *dep_name == member.name() && contains(*dep_feature) {
+                            // member/feat where "feat" is a feature in member.
+                            //
+                            // `weak` can be ignored here, because the member
+                            // either is or isn't being built.
+                            features.insert(FeatureValue::Feature(*dep_feature));
+                            found.insert(feature.clone());
+                        }
                     }
-                } else if contains(*feature) {
-                    // feature exists in this member.
-                    features.insert(*feature);
-                    found.insert(*feature);
                 }
             }
-            RequestedFeatures {
+            CliFeatures {
                 features: Rc::new(features),
                 all_features: false,
-                uses_default_features: requested_features.uses_default_features,
+                uses_default_features: cli_features.uses_default_features,
             }
         };
 
-        let members: Vec<(&Package, RequestedFeatures)> = self
+        let members: Vec<(&Package, CliFeatures)> = self
             .members()
             .filter(|m| specs.iter().any(|spec| spec.matches(m.package_id())))
             .map(|m| (m, matching_features(m)))
@@ -1025,27 +1161,28 @@ impl<'cfg> Workspace<'cfg> {
         if members.is_empty() {
             // `cargo build -p foo`, where `foo` is not a member.
             // Do not allow any command-line flags (defaults only).
-            if !(requested_features.features.is_empty()
-                && !requested_features.all_features
-                && requested_features.uses_default_features)
+            if !(cli_features.features.is_empty()
+                && !cli_features.all_features
+                && cli_features.uses_default_features)
             {
-                anyhow::bail!("cannot specify features for packages outside of workspace");
+                bail!("cannot specify features for packages outside of workspace");
             }
             // Add all members from the workspace so we can ensure `-p nonmember`
             // is in the resolve graph.
             return Ok(self
                 .members()
-                .map(|m| (m, RequestedFeatures::new_all(false)))
+                .map(|m| (m, CliFeatures::new_all(false)))
                 .collect());
         }
-        if *requested_features.features != found {
-            let missing: Vec<_> = requested_features
+        if *cli_features.features != found {
+            let mut missing: Vec<_> = cli_features
                 .features
                 .difference(&found)
-                .copied()
+                .map(|fv| fv.to_string())
                 .collect();
+            missing.sort();
             // TODO: typo suggestions would be good here.
-            anyhow::bail!(
+            bail!(
                 "none of the selected packages contains these features: {}",
                 missing.join(", ")
             );
@@ -1053,30 +1190,88 @@ impl<'cfg> Workspace<'cfg> {
         Ok(members)
     }
 
-    /// This is the current "stable" behavior for command-line feature selection.
-    fn members_with_features_stable(
+    /// This is the "old" behavior for command-line feature selection.
+    /// See `allows_new_cli_feature_behavior`.
+    fn members_with_features_old(
         &self,
         specs: &[PackageIdSpec],
-        requested_features: &RequestedFeatures,
-    ) -> CargoResult<Vec<(&Package, RequestedFeatures)>> {
+        cli_features: &CliFeatures,
+    ) -> Vec<(&Package, CliFeatures)> {
+        // Split off any features with the syntax `member-name/feature-name` into a map
+        // so that those features can be applied directly to those workspace-members.
+        let mut member_specific_features: HashMap<InternedString, BTreeSet<FeatureValue>> =
+            HashMap::new();
+        // Features for the member in the current directory.
+        let mut cwd_features = BTreeSet::new();
+        for feature in cli_features.features.iter() {
+            match feature {
+                FeatureValue::Feature(_) => {
+                    cwd_features.insert(feature.clone());
+                }
+                // This should be enforced by CliFeatures.
+                FeatureValue::Dep { .. }
+                | FeatureValue::DepFeature {
+                    dep_prefix: true, ..
+                } => panic!("unexpected dep: syntax {}", feature),
+                FeatureValue::DepFeature {
+                    dep_name,
+                    dep_feature,
+                    dep_prefix: _,
+                    weak: _,
+                } => {
+                    // I think weak can be ignored here.
+                    // * With `--features member?/feat -p member`, the ? doesn't
+                    //   really mean anything (either the member is built or it isn't).
+                    // * With `--features nonmember?/feat`, cwd_features will
+                    //   handle processing it correctly.
+                    let is_member = self.members().any(|member| member.name() == *dep_name);
+                    if is_member && specs.iter().any(|spec| spec.name() == *dep_name) {
+                        member_specific_features
+                            .entry(*dep_name)
+                            .or_default()
+                            .insert(FeatureValue::Feature(*dep_feature));
+                    } else {
+                        cwd_features.insert(feature.clone());
+                    }
+                }
+            }
+        }
+
         let ms = self.members().filter_map(|member| {
             let member_id = member.package_id();
             match self.current_opt() {
                 // The features passed on the command-line only apply to
                 // the "current" package (determined by the cwd).
                 Some(current) if member_id == current.package_id() => {
-                    Some((member, requested_features.clone()))
+                    let feats = CliFeatures {
+                        features: Rc::new(cwd_features.clone()),
+                        all_features: cli_features.all_features,
+                        uses_default_features: cli_features.uses_default_features,
+                    };
+                    Some((member, feats))
                 }
                 _ => {
                     // Ignore members that are not enabled on the command-line.
                     if specs.iter().any(|spec| spec.matches(member_id)) {
-                        // -p for a workspace member that is not the
-                        // "current" one, don't use the local
-                        // `--features`, only allow `--all-features`.
-                        Some((
-                            member,
-                            RequestedFeatures::new_all(requested_features.all_features),
-                        ))
+                        // -p for a workspace member that is not the "current"
+                        // one.
+                        //
+                        // The odd behavior here is due to backwards
+                        // compatibility. `--features` and
+                        // `--no-default-features` used to only apply to the
+                        // "current" package. As an extension, this allows
+                        // member-name/feature-name to set member-specific
+                        // features, which should be backwards-compatible.
+                        let feats = CliFeatures {
+                            features: Rc::new(
+                                member_specific_features
+                                    .remove(member.name().as_str())
+                                    .unwrap_or_default(),
+                            ),
+                            uses_default_features: true,
+                            all_features: cli_features.all_features,
+                        };
+                        Some((member, feats))
                     } else {
                         // This member was not requested on the command-line, skip.
                         None
@@ -1084,7 +1279,7 @@ impl<'cfg> Workspace<'cfg> {
                 }
             }
         });
-        Ok(ms.collect())
+        ms.collect()
     }
 }
 
@@ -1227,12 +1422,9 @@ impl WorkspaceRootConfig {
             Some(p) => p,
             None => return Ok(Vec::new()),
         };
-        let res =
-            glob(path).chain_err(|| anyhow::format_err!("could not parse pattern `{}`", &path))?;
+        let res = glob(path).with_context(|| format!("could not parse pattern `{}`", &path))?;
         let res = res
-            .map(|p| {
-                p.chain_err(|| anyhow::format_err!("unable to match path to pattern `{}`", &path))
-            })
+            .map(|p| p.with_context(|| format!("unable to match path to pattern `{}`", &path)))
             .collect::<Result<Vec<_>, _>>()?;
         Ok(res)
     }
