@@ -1,34 +1,12 @@
-#![cfg_attr(test, deny(warnings))]
-// While we're getting used to 2018:
+// For various reasons, some idioms are still allow'ed, but we would like to
+// test and enforce them.
 #![warn(rust_2018_idioms)]
-// Clippy isn't enforced by CI (@alexcrichton isn't a fan).
-#![allow(clippy::blacklisted_name)] // frequently used in tests
-#![allow(clippy::cognitive_complexity)] // large project
-#![allow(clippy::derive_hash_xor_eq)] // there's an intentional incoherence
-#![allow(clippy::explicit_into_iter_loop)] // explicit loops are clearer
-#![allow(clippy::explicit_iter_loop)] // explicit loops are clearer
-#![allow(clippy::identity_op)] // used for vertical alignment
-#![allow(clippy::implicit_hasher)] // large project
-#![allow(clippy::large_enum_variant)] // large project
-#![allow(clippy::new_without_default)] // explicit is maybe clearer
-#![allow(clippy::redundant_closure)] // closures can be less verbose
-#![allow(clippy::redundant_closure_call)] // closures over try catch blocks
-#![allow(clippy::too_many_arguments)] // large project
-#![allow(clippy::type_complexity)] // there's an exceptionally complex type
-#![allow(clippy::wrong_self_convention)] // perhaps `Rc` should be special-cased in Clippy?
-#![allow(clippy::write_with_newline)] // too pedantic
-#![allow(clippy::inefficient_to_string)] // this causes suggestions that result in `(*s).to_string()`
-#![allow(clippy::collapsible_if)] // too pedantic
+#![cfg_attr(test, deny(warnings))]
+// Due to some of the default clippy lints being somewhat subjective and not
+// necessarily an improvement, we prefer to not use them at this time.
+#![allow(clippy::all)]
 #![warn(clippy::needless_borrow)]
-// Unit is now interned, and would probably be better as pass-by-copy, but
-// doing so causes a lot of & and * shenanigans that makes the code arguably
-// less clear and harder to read.
-#![allow(clippy::trivially_copy_pass_by_ref)]
-// exhaustively destructuring ensures future fields are handled
-#![allow(clippy::unneeded_field_pattern)]
-// false positives in target-specific code, for details see
-// https://github.com/rust-lang/cargo/pull/7251#pullrequestreview-274914270
-#![allow(clippy::useless_conversion)]
+#![warn(clippy::redundant_clone)]
 
 use crate::core::shell::Verbosity::Verbose;
 use crate::core::Shell;
@@ -37,7 +15,7 @@ use log::debug;
 use std::fmt;
 
 pub use crate::util::errors::{InternalError, VerboseError};
-pub use crate::util::{CargoResult, CliError, CliResult, Config};
+pub use crate::util::{indented_lines, CargoResult, CliError, CliResult, Config};
 
 pub const CARGO_ENV: &str = "CARGO";
 
@@ -163,13 +141,11 @@ fn _display_error(err: &Error, shell: &mut Shell, as_err: bool) -> bool {
             return true;
         }
         drop(writeln!(shell.err(), "\nCaused by:"));
-        for line in cause.to_string().lines() {
-            if line.is_empty() {
-                drop(writeln!(shell.err()));
-            } else {
-                drop(writeln!(shell.err(), "  {}", line));
-            }
-        }
+        drop(write!(
+            shell.err(),
+            "{}",
+            indented_lines(&cause.to_string())
+        ));
     }
     false
 }
