@@ -1,5 +1,5 @@
 use crate::core::compiler::RustcTargetData;
-use crate::core::resolver::{features::RequestedFeatures, HasDevUnits, ResolveOpts};
+use crate::core::resolver::HasDevUnits;
 use crate::core::{Shell, Workspace};
 use crate::ops;
 use crate::util::CargoResult;
@@ -19,20 +19,12 @@ pub struct DocOptions {
 /// Main method for `cargo doc`.
 pub fn doc(ws: &Workspace<'_>, options: &DocOptions) -> CargoResult<()> {
     let specs = options.compile_opts.spec.to_package_id_specs(ws)?;
-    let opts = ResolveOpts::new(
-        /*dev_deps*/ true,
-        RequestedFeatures::from_command_line(
-            &options.compile_opts.features,
-            options.compile_opts.all_features,
-            !options.compile_opts.no_default_features,
-        ),
-    );
     let target_data = RustcTargetData::new(ws, &options.compile_opts.build_config.requested_kinds)?;
     let ws_resolve = ops::resolve_ws_with_opts(
         ws,
         &target_data,
         &options.compile_opts.build_config.requested_kinds,
-        &opts,
+        &options.compile_opts.cli_features,
         &specs,
         HasDevUnits::No,
         crate::core::resolver::features::ForceAllTargets::No,
