@@ -3,15 +3,16 @@ use std::env;
 use std::io::prelude::*;
 use std::io::SeekFrom;
 use std::path::{Path, PathBuf};
+use std::rc::Rc;
 
-use anyhow::{bail, format_err};
+use anyhow::{bail, format_err, Context as _};
 use serde::{Deserialize, Serialize};
 
 use crate::core::compiler::Freshness;
-use crate::core::{Dependency, Package, PackageId, Source, SourceId};
+use crate::core::{Dependency, FeatureValue, Package, PackageId, Source, SourceId};
 use crate::ops::{self, CompileFilter, CompileOptions};
 use crate::sources::PathSource;
-use crate::util::errors::{CargoResult, CargoResultExt};
+use crate::util::errors::CargoResult;
 use crate::util::Config;
 use crate::util::{FileLock, Filesystem};
 
@@ -100,12 +101,11 @@ impl InstallTracker {
             if contents.is_empty() {
                 Ok(CrateListingV1::default())
             } else {
-                Ok(toml::from_str(&contents)
-                    .chain_err(|| format_err!("invalid TOML found for metadata"))?)
+                Ok(toml::from_str(&contents).with_context(|| "invalid TOML found for metadata")?)
             }
         })()
-        .chain_err(|| {
-            format_err!(
+        .with_context(|| {
+            format!(
                 "failed to parse crate metadata at `{}`",
                 v1_lock.path().to_string_lossy()
             )
@@ -118,13 +118,13 @@ impl InstallTracker {
                 CrateListingV2::default()
             } else {
                 serde_json::from_str(&contents)
-                    .chain_err(|| format_err!("invalid JSON found for metadata"))?
+                    .with_context(|| "invalid JSON found for metadata")?
             };
-            v2.sync_v1(&v1)?;
+            v2.sync_v1(&v1);
             Ok(v2)
         })()
-        .chain_err(|| {
-            format_err!(
+        .with_context(|| {
+            format!(
                 "failed to parse crate metadata at `{}`",
                 v2_lock.path().to_string_lossy()
             )
@@ -277,15 +277,15 @@ impl InstallTracker {
 
     /// Save tracking information to disk.
     pub fn save(&self) -> CargoResult<()> {
-        self.v1.save(&self.v1_lock).chain_err(|| {
-            format_err!(
+        self.v1.save(&self.v1_lock).with_context(|| {
+            format!(
                 "failed to write crate metadata at `{}`",
                 self.v1_lock.path().to_string_lossy()
             )
         })?;
 
-        self.v2.save(&self.v2_lock).chain_err(|| {
-            format_err!(
+        self.v2.save(&self.v2_lock).with_context(|| {
+            format!(
                 "failed to write crate metadata at `{}`",
                 self.v2_lock.path().to_string_lossy()
             )
@@ -367,7 +367,7 @@ impl CrateListingV2 {
     /// where v2 is in use, and a v1 update is made, then v2 is used again.
     /// i.e., `cargo +new install foo ; cargo +old install bar ; cargo +new install bar`
     /// For now, v1 is the source of truth, so its values are trusted over v2.
-    fn sync_v1(&mut self, v1: &CrateListingV1) -> CargoResult<()> {
+    fn sync_v1(&mut self, v1: &CrateListingV1) {
         // Make the `bins` entries the same.
         for (pkg_id, bins) in &v1.v1 {
             self.installs
@@ -385,7 +385,6 @@ impl CrateListingV2 {
         for pkg_id in to_remove {
             self.installs.remove(&pkg_id);
         }
-        Ok(())
     }
 
     fn package_for_bin(&self, bin_name: &str) -> Option<PackageId> {
@@ -423,9 +422,9 @@ impl CrateListingV2 {
         if let Some(info) = self.installs.get_mut(&pkg.package_id()) {
             info.bins.append(&mut bins.clone());
             info.version_req = version_req;
-            info.features = feature_set(&opts.features);
-            info.all_features = opts.all_features;
-            info.no_default_features = opts.no_default_features;
+            info.features = feature_set(&opts.cli_features.features);
+            info.all_features = opts.cli_features.all_features;
+            info.no_default_features = !opts.cli_features.uses_default_features;
             info.profile = opts.build_config.requested_profile.to_string();
             info.target = Some(target.to_string());
             info.rustc = Some(rustc.to_string());
@@ -435,9 +434,9 @@ impl CrateListingV2 {
                 InstallInfo {
                     version_req,
                     bins: bins.clone(),
-                    features: feature_set(&opts.features),
-                    all_features: opts.all_features,
-                    no_default_features: opts.no_default_features,
+                    features: feature_set(&opts.cli_features.features),
+                    all_features: opts.cli_features.all_features,
+                    no_default_features: !opts.cli_features.uses_default_features,
                     profile: opts.build_config.requested_profile.to_string(),
                     target: Some(target.to_string()),
                     rustc: Some(rustc.to_string()),
@@ -490,9 +489,9 @@ impl InstallInfo {
     ///
     /// This does not do Package/Source/Version checking.
     fn is_up_to_date(&self, opts: &CompileOptions, target: &str, exes: &BTreeSet<String>) -> bool {
-        self.features == feature_set(&opts.features)
-            && self.all_features == opts.all_features
-            && self.no_default_features == opts.no_default_features
+        self.features == feature_set(&opts.cli_features.features)
+            && self.all_features == opts.cli_features.all_features
+            && self.no_default_features != opts.cli_features.uses_default_features
             && self.profile.as_str() == opts.build_config.requested_profile.as_str()
             && (self.target.is_none() || self.target.as_deref() == Some(target))
             && &self.bins == exes
@@ -642,9 +641,9 @@ where
     }
 }
 
-/// Helper to convert features Vec to a BTreeSet.
-fn feature_set(features: &[String]) -> BTreeSet<String> {
-    features.iter().cloned().collect()
+/// Helper to convert features to a BTreeSet.
+fn feature_set(features: &Rc<BTreeSet<FeatureValue>>) -> BTreeSet<String> {
+    features.iter().map(|s| s.to_string()).collect()
 }
 
 /// Helper to get the executable names from a filter.
