@@ -11,18 +11,21 @@
 //!   providing the most power and flexibility.
 
 use crate::core::compiler::{CompileKind, RustcTargetData};
-use crate::core::registry::PackageRegistry;
+use crate::core::registry::{LockedPatchDependency, PackageRegistry};
 use crate::core::resolver::features::{
-    FeatureOpts, FeatureResolver, ForceAllTargets, ResolvedFeatures,
+    CliFeatures, FeatureOpts, FeatureResolver, ForceAllTargets, RequestedFeatures, ResolvedFeatures,
 };
-use crate::core::resolver::{self, HasDevUnits, Resolve, ResolveOpts};
+use crate::core::resolver::{self, HasDevUnits, Resolve, ResolveOpts, ResolveVersion};
 use crate::core::summary::Summary;
 use crate::core::Feature;
-use crate::core::{PackageId, PackageIdSpec, PackageSet, Source, SourceId, Workspace};
+use crate::core::{
+    GitReference, PackageId, PackageIdSpec, PackageSet, Source, SourceId, Workspace,
+};
 use crate::ops;
 use crate::sources::PathSource;
-use crate::util::errors::{CargoResult, CargoResultExt};
+use crate::util::errors::CargoResult;
 use crate::util::{profile, CanonicalUrl};
+use anyhow::Context as _;
 use log::{debug, trace};
 use std::collections::HashSet;
 
@@ -76,9 +79,9 @@ pub fn resolve_ws<'a>(ws: &Workspace<'a>) -> CargoResult<(PackageSet<'a>, Resolv
 /// members. In this case, `opts.all_features` must be `true`.
 pub fn resolve_ws_with_opts<'cfg>(
     ws: &Workspace<'cfg>,
-    target_data: &RustcTargetData,
+    target_data: &RustcTargetData<'cfg>,
     requested_targets: &[CompileKind],
-    opts: &ResolveOpts,
+    cli_features: &CliFeatures,
     specs: &[PackageIdSpec],
     has_dev_units: HasDevUnits,
     force_all_targets: ForceAllTargets,
@@ -120,7 +123,8 @@ pub fn resolve_ws_with_opts<'cfg>(
     let resolved_with_overrides = resolve_with_previous(
         &mut registry,
         ws,
-        opts,
+        cli_features,
+        has_dev_units,
         resolve.as_ref(),
         None,
         specs,
@@ -130,7 +134,7 @@ pub fn resolve_ws_with_opts<'cfg>(
     let pkg_set = get_resolved_packages(&resolved_with_overrides, registry)?;
 
     let member_ids = ws
-        .members_with_features(specs, &opts.features)?
+        .members_with_features(specs, cli_features)?
         .into_iter()
         .map(|(p, _fts)| p.package_id())
         .collect::<Vec<_>>();
@@ -149,7 +153,7 @@ pub fn resolve_ws_with_opts<'cfg>(
         target_data,
         &resolved_with_overrides,
         &pkg_set,
-        &opts.features,
+        cli_features,
         specs,
         requested_targets,
         feature_opts,
@@ -171,7 +175,8 @@ fn resolve_with_registry<'cfg>(
     let mut resolve = resolve_with_previous(
         registry,
         ws,
-        &ResolveOpts::everything(),
+        &CliFeatures::new_all(true),
+        HasDevUnits::Yes,
         prev.as_ref(),
         None,
         &[],
@@ -202,7 +207,8 @@ fn resolve_with_registry<'cfg>(
 pub fn resolve_with_previous<'cfg>(
     registry: &mut PackageRegistry<'cfg>,
     ws: &Workspace<'cfg>,
-    opts: &ResolveOpts,
+    cli_features: &CliFeatures,
+    has_dev_units: HasDevUnits,
     previous: Option<&Resolve>,
     to_avoid: Option<&HashSet<PackageId>>,
     specs: &[PackageIdSpec],
@@ -251,26 +257,91 @@ pub fn resolve_with_previous<'cfg>(
                     continue;
                 }
             };
-            let patches = patches
-                .iter()
-                .map(|dep| {
-                    let unused = previous.unused_patches().iter().cloned();
-                    let candidates = previous.iter().chain(unused);
-                    match candidates
-                        .filter(pre_patch_keep)
-                        .find(|&id| dep.matches_id(id))
-                    {
-                        Some(id) => {
-                            let mut locked_dep = dep.clone();
-                            locked_dep.lock_to(id);
-                            (dep, Some((locked_dep, id)))
-                        }
-                        None => (dep, None),
+
+            // This is a list of pairs where the first element of the pair is
+            // the raw `Dependency` which matches what's listed in `Cargo.toml`.
+            // The second element is, if present, the "locked" version of
+            // the `Dependency` as well as the `PackageId` that it previously
+            // resolved to. This second element is calculated by looking at the
+            // previous resolve graph, which is primarily what's done here to
+            // build the `registrations` list.
+            let mut registrations = Vec::new();
+            for dep in patches {
+                let candidates = || {
+                    previous
+                        .iter()
+                        .chain(previous.unused_patches().iter().cloned())
+                        .filter(&pre_patch_keep)
+                };
+
+                let lock = match candidates().find(|id| dep.matches_id(*id)) {
+                    // If we found an exactly matching candidate in our list of
+                    // candidates, then that's the one to use.
+                    Some(package_id) => {
+                        let mut locked_dep = dep.clone();
+                        locked_dep.lock_to(package_id);
+                        Some(LockedPatchDependency {
+                            dependency: locked_dep,
+                            package_id,
+                            alt_package_id: None,
+                        })
                     }
-                })
-                .collect::<Vec<_>>();
+                    None => {
+                        // If the candidate does not have a matching source id
+                        // then we may still have a lock candidate. If we're
+                        // loading a v2-encoded resolve graph and `dep` is a
+                        // git dep with `branch = 'master'`, then this should
+                        // also match candidates without `branch = 'master'`
+                        // (which is now treated separately in Cargo).
+                        //
+                        // In this scenario we try to convert candidates located
+                        // in the resolve graph to explicitly having the
+                        // `master` branch (if they otherwise point to
+                        // `DefaultBranch`). If this works and our `dep`
+                        // matches that then this is something we'll lock to.
+                        match candidates().find(|&id| {
+                            match master_branch_git_source(id, previous) {
+                                Some(id) => dep.matches_id(id),
+                                None => false,
+                            }
+                        }) {
+                            Some(id_using_default) => {
+                                let id_using_master = id_using_default.with_source_id(
+                                    dep.source_id().with_precise(
+                                        id_using_default
+                                            .source_id()
+                                            .precise()
+                                            .map(|s| s.to_string()),
+                                    ),
+                                );
+
+                                let mut locked_dep = dep.clone();
+                                locked_dep.lock_to(id_using_master);
+                                Some(LockedPatchDependency {
+                                    dependency: locked_dep,
+                                    package_id: id_using_master,
+                                    // Note that this is where the magic
+                                    // happens, where the resolve graph
+                                    // probably has locks pointing to
+                                    // DefaultBranch sources, and by including
+                                    // this here those will get transparently
+                                    // rewritten to Branch("master") which we
+                                    // have a lock entry for.
+                                    alt_package_id: Some(id_using_default),
+                                })
+                            }
+
+                            // No locked candidate was found
+                            None => None,
+                        }
+                    }
+                };
+
+                registrations.push((dep, lock));
+            }
+
             let canonical = CanonicalUrl::new(url)?;
-            for (orig_patch, unlock_id) in registry.patch(url, &patches)? {
+            for (orig_patch, unlock_id) in registry.patch(url, &registrations)? {
                 // Avoid the locked patch ID.
                 avoid_patch_ids.insert(unlock_id);
                 // Also avoid the thing it is patching.
@@ -285,12 +356,13 @@ pub fn resolve_with_previous<'cfg>(
 
     let keep = |p: &PackageId| pre_patch_keep(p) && !avoid_patch_ids.contains(p);
 
+    let dev_deps = ws.require_optional_deps() || has_dev_units == HasDevUnits::Yes;
     // In the case where a previous instance of resolve is available, we
     // want to lock as many packages as possible to the previous version
     // without disturbing the graph structure.
     if let Some(r) = previous {
         trace!("previous: {:?}", r);
-        register_previous_locks(ws, registry, r, &keep);
+        register_previous_locks(ws, registry, r, &keep, dev_deps);
     }
     // Everything in the previous lock file we want to keep is prioritized
     // in dependency selection if it comes up, aka we want to have
@@ -315,15 +387,15 @@ pub fn resolve_with_previous<'cfg>(
     }
 
     let summaries: Vec<(Summary, ResolveOpts)> = ws
-        .members_with_features(specs, &opts.features)?
+        .members_with_features(specs, cli_features)?
         .into_iter()
         .map(|(member, features)| {
             let summary = registry.lock(member.summary().clone());
             (
                 summary,
                 ResolveOpts {
-                    dev_deps: opts.dev_deps,
-                    features,
+                    dev_deps,
+                    features: RequestedFeatures::CliFeatures(features),
                 },
             )
         })
@@ -406,7 +478,7 @@ pub fn add_overrides<'a>(
     for (path, definition) in paths {
         let id = SourceId::for_path(&path)?;
         let mut source = PathSource::new_recursive(&path, id, ws.config());
-        source.update().chain_err(|| {
+        source.update().with_context(|| {
             format!(
                 "failed to update path override `{}` \
                  (defined in `{}`)",
@@ -448,6 +520,7 @@ fn register_previous_locks(
     registry: &mut PackageRegistry<'_>,
     resolve: &Resolve,
     keep: &dyn Fn(&PackageId) -> bool,
+    dev_deps: bool,
 ) {
     let path_pkg = |id: SourceId| {
         if !id.is_path() {
@@ -557,6 +630,11 @@ fn register_previous_locks(
                 continue;
             }
 
+            // If dev-dependencies aren't being resolved, skip them.
+            if !dep.is_transitive() && !dev_deps {
+                continue;
+            }
+
             // If this is a path dependency, then try to push it onto our
             // worklist.
             if let Some(pkg) = path_pkg(dep.source_id()) {
@@ -599,7 +677,22 @@ fn register_previous_locks(
             .deps_not_replaced(node)
             .map(|p| p.0)
             .filter(keep)
-            .collect();
+            .collect::<Vec<_>>();
+
+        // In the v2 lockfile format and prior the `branch=master` dependency
+        // directive was serialized the same way as the no-branch-listed
+        // directive. Nowadays in Cargo, however, these two directives are
+        // considered distinct and are no longer represented the same way. To
+        // maintain compatibility with older lock files we register locked nodes
+        // for *both* the master branch and the default branch.
+        //
+        // Note that this is only applicable for loading older resolves now at
+        // this point. All new lock files are encoded as v3-or-later, so this is
+        // just compat for loading an old lock file successfully.
+        if let Some(node) = master_branch_git_source(node, resolve) {
+            registry.register_lock(node, deps.clone());
+        }
+
         registry.register_lock(node, deps);
     }
 
@@ -614,3 +707,17 @@ fn register_previous_locks(
         }
     }
 }
+
+fn master_branch_git_source(id: PackageId, resolve: &Resolve) -> Option<PackageId> {
+    if resolve.version() <= ResolveVersion::V2 {
+        let source = id.source_id();
+        if let Some(GitReference::DefaultBranch) = source.git_reference() {
+            let new_source =
+                SourceId::for_git(source.url(), GitReference::Branch("master".to_string()))
+                    .unwrap()
+                    .with_precise(source.precise().map(|s| s.to_string()));
+            return Some(id.with_source_id(new_source));
+        }
+    }
+    None
+}
