@@ -1,11 +1,11 @@
 use crate::util::errors::CargoResult;
-use crate::util::paths;
+use cargo_util::paths;
 use std::path::{Path, PathBuf};
 
 /// Finds the root `Cargo.toml`.
 pub fn find_root_manifest_for_wd(cwd: &Path) -> CargoResult<PathBuf> {
     let file = "Cargo.toml";
-    for current in paths::ancestors(cwd) {
+    for current in paths::ancestors(cwd, None) {
         let manifest = current.join(file);
         if manifest.exists() {
             return Ok(manifest);
