@@ -1,12 +1,12 @@
+use std::fmt;
 use std::time::Duration;
 
 pub use self::canonical_url::CanonicalUrl;
 pub use self::config::{homedir, Config, ConfigValue};
 pub use self::dependency_queue::DependencyQueue;
 pub use self::diagnostic_server::RustfixDiagnosticServer;
-pub use self::errors::{exit_status_to_string, internal, process_error, process_error_raw};
-pub use self::errors::{CargoResult, CargoResultExt, CliResult, Test};
-pub use self::errors::{CargoTestError, CliError, ProcessError};
+pub use self::errors::{internal, CargoResult, CliResult, Test};
+pub use self::errors::{CargoTestError, CliError};
 pub use self::flock::{FileLock, Filesystem};
 pub use self::graph::Graph;
 pub use self::hasher::StableHasher;
@@ -15,15 +15,10 @@ pub use self::into_url::IntoUrl;
 pub use self::into_url_with_base::IntoUrlWithBase;
 pub use self::lev_distance::{closest, closest_msg, lev_distance};
 pub use self::lockserver::{LockServer, LockServerClient, LockServerStarted};
-pub use self::paths::{bytes2path, dylib_path, join_paths, path2bytes};
-pub use self::paths::{dylib_path_envvar, normalize_path};
-pub use self::process_builder::{process, ProcessBuilder};
 pub use self::progress::{Progress, ProgressStyle};
 pub use self::queue::Queue;
-pub use self::read2::read2;
 pub use self::restricted_names::validate_package_name;
 pub use self::rustc::Rustc;
-pub use self::sha256::Sha256;
 pub use self::to_semver::ToSemver;
 pub use self::vcs::{existing_vcs_repo, FossilRepo, GitRepo, HgRepo, PijulRepo};
 pub use self::workspace::{
@@ -51,15 +46,11 @@ pub mod lev_distance;
 mod lockserver;
 pub mod machine_message;
 pub mod network;
-pub mod paths;
-pub mod process_builder;
 pub mod profile;
 mod progress;
 mod queue;
-mod read2;
 pub mod restricted_names;
 pub mod rustc;
-mod sha256;
 pub mod to_semver;
 pub mod toml;
 mod vcs;
@@ -75,9 +66,30 @@ pub fn elapsed(duration: Duration) -> String {
     }
 }
 
-/// Whether or not this running in a Continuous Integration environment.
-pub fn is_ci() -> bool {
-    std::env::var("CI").is_ok() || std::env::var("TF_BUILD").is_ok()
+pub fn iter_join_onto<W, I, T>(mut w: W, iter: I, delim: &str) -> fmt::Result
+where
+    W: fmt::Write,
+    I: IntoIterator<Item = T>,
+    T: std::fmt::Display,
+{
+    let mut it = iter.into_iter().peekable();
+    while let Some(n) = it.next() {
+        write!(w, "{}", n)?;
+        if it.peek().is_some() {
+            write!(w, "{}", delim)?;
+        }
+    }
+    Ok(())
+}
+
+pub fn iter_join<I, T>(iter: I, delim: &str) -> String
+where
+    I: IntoIterator<Item = T>,
+    T: std::fmt::Display,
+{
+    let mut s = String::new();
+    let _ = iter_join_onto(&mut s, iter, delim);
+    s
 }
 
 pub fn indented_lines(text: &str) -> String {
