@@ -94,7 +94,7 @@ fn bad4() {
             ".cargo/config",
             r#"
                 [cargo-new]
-                  name = false
+                  vcs = false
             "#,
         )
         .build();
@@ -105,7 +105,7 @@ fn bad4() {
 [ERROR] Failed to create package `foo` at `[..]`
 
 Caused by:
-  error in [..]config: `cargo-new.name` expected a string, but found a boolean
+  error in [..]config: `cargo-new.vcs` expected a string, but found a boolean
 ",
         )
         .run();
@@ -1309,7 +1309,7 @@ fn bad_debuginfo() {
 error: failed to parse manifest at `[..]`
 
 Caused by:
-  invalid type: string \"a\", expected a boolean or an integer for [..]
+  expected a boolean or an integer for [..]
 ",
         )
         .run();
@@ -1338,7 +1338,7 @@ fn bad_opt_level() {
 error: failed to parse manifest at `[..]`
 
 Caused by:
-  invalid type: integer `3`, expected a boolean or a string for key [..]
+  expected a boolean or a string for key [..]
 ",
         )
         .run();
@@ -1391,16 +1391,16 @@ fn bad_target_cfg() {
         .with_stderr(
             "\
 [ERROR] error in [..]/foo/.cargo/config: \
-could not load config key `target.cfg(not(target_os = \"none\")).runner`
+could not load config key `target.\"cfg(not(target_os = /\"none/\"))\".runner`
 
 Caused by:
   error in [..]/foo/.cargo/config: \
-  could not load config key `target.cfg(not(target_os = \"none\")).runner`
+  could not load config key `target.\"cfg(not(target_os = /\"none/\"))\".runner`
 
 Caused by:
-  invalid configuration for key `target.cfg(not(target_os = \"none\")).runner`
+  invalid configuration for key `target.\"cfg(not(target_os = /\"none/\"))\".runner`
   expected a string or array of strings, but found a boolean for \
-  `target.cfg(not(target_os = \"none\")).runner` in [..]/foo/.cargo/config
+  `target.\"cfg(not(target_os = /\"none/\"))\".runner` in [..]/foo/.cargo/config
 ",
         )
         .run();
