@@ -1301,7 +1301,7 @@ fn dev_deps_with_testing() {
 [COMPILING] [..] v0.5.0 ([..])
 [COMPILING] [..] v0.5.0 ([..]
 [FINISHED] test [unoptimized + debuginfo] target(s) in [..]
-[RUNNING] target/debug/deps/foo-[..][EXE]",
+[RUNNING] [..] (target/debug/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test tests::foo ... ok")
         .run();
@@ -2805,7 +2805,7 @@ fn default_not_master() {
 
     // Then create a commit on the new `main` branch so `master` and `main`
     // differ.
-    git_project.change_file("src/lib.rs", "");
+    git_project.change_file("src/lib.rs", "pub fn bar() {}");
     git::add(&repo);
     git::commit(&repo);
 
@@ -2817,14 +2817,13 @@ fn default_not_master() {
                     [project]
                     name = "foo"
                     version = "0.5.0"
-
                     [dependencies]
                     dep1 = {{ git = '{}' }}
                 "#,
                 git_project.url()
             ),
         )
-        .file("src/lib.rs", "pub fn foo() { dep1::foo() }")
+        .file("src/lib.rs", "pub fn foo() { dep1::bar() }")
         .build();
 
     project
@@ -2832,14 +2831,6 @@ fn default_not_master() {
         .with_stderr(
             "\
 [UPDATING] git repository `[..]`
-warning: fetching `master` branch from `[..]` but the `HEAD` \
-    reference for this repository is not the \
-    `master` branch. This behavior will change \
-    in Cargo in the future and your build may \
-    break, so it's recommended to place \
-    `branch = \"master\"` in Cargo.toml when \
-    depending on this git repository to ensure \
-    that your build will continue to work.
 [COMPILING] dep1 v0.5.0 ([..])
 [COMPILING] foo v0.5.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]",
@@ -2977,7 +2968,6 @@ fn two_dep_forms() {
                     [project]
                     name = "foo"
                     version = "0.5.0"
-
                     [dependencies]
                     dep1 = {{ git = '{}', branch = 'master' }}
                     a = {{ path = 'a' }}
@@ -2993,7 +2983,6 @@ fn two_dep_forms() {
                     [project]
                     name = "a"
                     version = "0.5.0"
-
                     [dependencies]
                     dep1 = {{ git = '{}' }}
                 "#,
@@ -3003,15 +2992,16 @@ fn two_dep_forms() {
         .file("a/src/lib.rs", "")
         .build();
 
+    // This'll download the git repository twice, one with HEAD and once with
+    // the master branch. Then it'll compile 4 crates, the 2 git deps, then
+    // the two local deps.
     project
         .cargo("build")
         .with_stderr(
             "\
 [UPDATING] [..]
-warning: two git dependencies found for `[..]` where one uses `branch = \"master\"` \
-and the other doesn't; this will break in a future version of Cargo, so please \
-ensure the dependency forms are consistent
-warning: [..]
+[UPDATING] [..]
+[COMPILING] [..]
 [COMPILING] [..]
 [COMPILING] [..]
 [COMPILING] [..]
