@@ -509,6 +509,11 @@ fn strip_works() {
 
 #[cargo_test]
 fn strip_requires_cargo_feature() {
+    if !is_nightly() {
+        // -Zstrip is unstable
+        return;
+    }
+
     let p = project()
         .file(
             "Cargo.toml",
@@ -541,7 +546,12 @@ Caused by:
 }
 
 #[cargo_test]
-fn strip_rejects_invalid_option() {
+fn strip_passes_unknown_option_to_rustc() {
+    if !is_nightly() {
+        // -Zstrip is unstable
+        return;
+    }
+
     let p = project()
         .file(
             "Cargo.toml",
@@ -553,7 +563,7 @@ fn strip_rejects_invalid_option() {
                 version = "0.1.0"
 
                 [profile.release]
-                strip = 'wrong'
+                strip = 'unknown'
             "#,
         )
         .file("src/main.rs", "fn main() {}")
@@ -562,13 +572,77 @@ fn strip_rejects_invalid_option() {
     p.cargo("build --release -v")
         .masquerade_as_nightly_cargo()
         .with_status(101)
-        .with_stderr(
+        .with_stderr_contains(
             "\
-[ERROR] failed to parse manifest at `[CWD]/Cargo.toml`
-
-Caused by:
-  unknown variant `wrong`, expected one of `debuginfo`, `none`, `symbols` for key [..]
+[COMPILING] foo [..]
+[RUNNING] `rustc [..] -Z strip=unknown [..]`
+error: incorrect value `unknown` for debugging option `strip` - either `none`, `debuginfo`, or `symbols` was expected
 ",
         )
         .run();
 }
+
+#[cargo_test]
+fn strip_accepts_true_to_strip_symbols() {
+    if !is_nightly() {
+        // -Zstrip is unstable
+        return;
+    }
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                cargo-features = ["strip"]
+
+                [package]
+                name = "foo"
+                version = "0.1.0"
+
+                [profile.release]
+                strip = true
+            "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build --release -v")
+        .masquerade_as_nightly_cargo()
+        .with_stderr(
+            "\
+[COMPILING] foo [..]
+[RUNNING] `rustc [..] -Z strip=symbols [..]`
+[FINISHED] [..]
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn strip_accepts_false_to_disable_strip() {
+    if !is_nightly() {
+        // -Zstrip is unstable
+        return;
+    }
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                cargo-features = ["strip"]
+
+                [package]
+                name = "foo"
+                version = "0.1.0"
+
+                [profile.release]
+                strip = false
+            "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build --release -v")
+        .masquerade_as_nightly_cargo()
+        .with_stderr_does_not_contain("-Z strip")
+        .run();
+}
