@@ -7,7 +7,7 @@
 use cargo_test_support::registry::{Dependency, Package};
 use cargo_test_support::ProjectBuilder;
 use cargo_test_support::{is_nightly, paths, project, rustc_host, Execs};
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 struct Setup {
     rustc_wrapper: PathBuf,
@@ -128,26 +128,12 @@ fn setup() -> Option<Setup> {
     })
 }
 
-fn enable_build_std(e: &mut Execs, setup: &Setup, arg: Option<&str>) {
+fn enable_build_std(e: &mut Execs, setup: &Setup) {
     // First up, force Cargo to use our "mock sysroot" which mimics what
     // libstd looks like upstream.
-    let root = paths::root();
-    let root = root
-        .parent() // chop off test name
-        .unwrap()
-        .parent() // chop off `citN`
-        .unwrap()
-        .parent() // chop off `target`
-        .unwrap()
-        .join("tests/testsuite/mock-std");
+    let root = Path::new(env!("CARGO_MANIFEST_DIR")).join("tests/testsuite/mock-std");
     e.env("__CARGO_TESTS_ONLY_SRC_ROOT", &root);
 
-    // Actually enable `-Zbuild-std` for now
-    let arg = match arg {
-        Some(s) => format!("-Zbuild-std={}", s),
-        None => "-Zbuild-std".to_string(),
-    };
-    e.arg(arg);
     e.masquerade_as_nightly_cargo();
 
     // We do various shenanigans to ensure our "mock sysroot" actually links
@@ -181,12 +167,14 @@ trait BuildStd: Sized {
 
 impl BuildStd for Execs {
     fn build_std(&mut self, setup: &Setup) -> &mut Self {
-        enable_build_std(self, setup, None);
+        enable_build_std(self, setup);
+        self.arg("-Zbuild-std");
         self
     }
 
     fn build_std_arg(&mut self, setup: &Setup, arg: &str) -> &mut Self {
-        enable_build_std(self, setup, Some(arg));
+        enable_build_std(self, setup);
+        self.arg(format!("-Zbuild-std={}", arg));
         self
     }
 
@@ -604,3 +592,73 @@ fn ignores_incremental() {
         .unwrap()
         .starts_with("foo-"));
 }
+
+#[cargo_test]
+fn cargo_config_injects_compiler_builtins() {
+    let setup = match setup() {
+        Some(s) => s,
+        None => return,
+    };
+    let p = project()
+        .file(
+            "src/lib.rs",
+            r#"
+                #![no_std]
+                pub fn foo() {
+                    assert_eq!(u8::MIN, 0);
+                }
+            "#,
+        )
+        .file(
+            ".cargo/config.toml",
+            r#"
+                [unstable]
+                build-std = ['core']
+            "#,
+        )
+        .build();
+    let mut build = p.cargo("build -v --lib");
+    enable_build_std(&mut build, &setup);
+    build
+        .target_host()
+        .with_stderr_does_not_contain("[..]libstd[..]")
+        .run();
+}
+
+#[cargo_test]
+fn different_features() {
+    let setup = match setup() {
+        Some(s) => s,
+        None => return,
+    };
+    let p = project()
+        .file(
+            "src/lib.rs",
+            "
+                pub fn foo() {
+                    std::conditional_function();
+                }
+            ",
+        )
+        .build();
+    p.cargo("build")
+        .build_std(&setup)
+        .arg("-Zbuild-std-features=feature1")
+        .target_host()
+        .run();
+}
+
+#[cargo_test]
+fn no_roots() {
+    // Checks for a bug where it would panic if there are no roots.
+    let setup = match setup() {
+        Some(s) => s,
+        None => return,
+    };
+    let p = project().file("tests/t1.rs", "").build();
+    p.cargo("build")
+        .build_std(&setup)
+        .target_host()
+        .with_stderr_contains("[FINISHED] [..]")
+        .run();
+}
